#! /bin/sh
# Copyright (c) 2019 Arista Networks, Inc.  All rights reserved.
# Arista Networks, Inc. Confidential and Proprietary.

RULE1='-p tcp -m tcpmss --mss 1:500 -j DROP'
PREFIX='Hotfix SecurityAdvisory0041:'

function sa41_versioncheck() {
	if [ -e /proc/sys/net/ipv4/tcp_min_snd_mss ]; then
		logger -t SECURITY-3-HOTFIX_ERR "$PREFIX Hotfix does not appear" \
			"valid for this version of EOS. Skipping install"
		exit 1
	fi
}

# insert rule after the one accepting established connections
function sa41_insertrule() {
	local iptablesBin=$1
	local rule=$2
	local establishedRuleId
	local ruleId

	if establishedRuleId=$(set -o pipefail && \
			    $iptablesBin -S INPUT 2>/dev/null| \
			    grep -n -- '-m conntrack' | \
			    grep -- '--ctstate \(\|[^ ]*,\)ESTABLISHED[, ]' | \
			    tail -n 1 | \
			    cut -f1 -d:); then
		ruleId=$establishedRuleId
	else
		ruleId=1
	fi

	$iptablesBin -I INPUT $ruleId $rule >&/dev/null
}

function sa41_selecttable() {
	# try to fetch first rule of INPUT chain in filter table
	# if filter table is not available, fall back to mangle
	if iptables -S INPUT 1 >/dev/null 2>&1; then
		echo "filter"
	else
		echo "mangle"
	fi
}

function sa41_nsinstall() {
	local ns="$2"
	local table="$3"

	# check
	if iptables -t ${table} -C INPUT $RULE1 >&/dev/null &&
	   ip6tables -t ${table} -C INPUT $RULE1 >&/dev/null; then
		test "$1" != "check" && logger -t SECURITY-6-HOTFIX_INFO \
			"$PREFIX Rules are already configured in $ns. Skipping install"
		return 0
	fi

	# at least one rule is missing, delete the other if it exists
	iptables -t ${table} -D INPUT $RULE1 2> /dev/null
	ip6tables -t ${table} -D INPUT $RULE1 2> /dev/null

	# re-add the rule
	sa41_insertrule "iptables -t ${table}" "$RULE1" &&
	sa41_insertrule "ip6tables -t ${table}" "$RULE1" &&
	logger -t SECURITY-6-HOTFIX_INFO "$PREFIX Rules have been installed in $ns" ||
	logger -t SECURITY-3-HOTFIX_ERR "$PREFIX Rules could not be installed in $ns"
}

function sa41_install() {
	sa41_versioncheck

	(
	if ! flock -n 9; then
		logger -t SECURITY-6-HOTFIX_INFO "$PREFIX Another instance is already running"
		exit 0
	fi
	local table=$(sa41_selecttable)

	for ns in $(ip netns list); do
		ip netns exec "$ns" $0 --nsinstall "$1" "$ns" "$table"
	done
	) 9>/var/lock/secAdv0041
}

function sa41_nsuninstall() {
	local table="$1"

	iptables -t ${table} -D INPUT $RULE1
	ip6tables -t ${table} -D INPUT $RULE1
}

function sa41_uninstall() {
	local table=$(sa41_selecttable)

	for ns in $(ip netns list); do
		ip netns exec "$ns" $0 --nsuninstall "$table"
	done
	logger -t SECURITY-6-HOTFIX_INFO "$PREFIX Hotfix uninstalled successfully"
}

function sa41_help() {
	cat <<EOF
$0 --install|--check|--uninstall - Deploy a hotfix for SecurityAdvisory0041
	--install Install hotfix
	--check Ensures hotfix is installed
	--uninstall Uninstall hotfix
EOF
	exit 1
}

case "$1" in
	--install) sa41_install;;
	--nsinstall) sa41_nsinstall "$2" "$3" "$4";;
	--check) sa41_install check;;
	--uninstall) sa41_uninstall;;
	--nsuninstall) sa41_nsuninstall "$2";;
	*) sa41_help;;
esac
